// =============================================================================
//
//   glPBuffer.pas
//
// =============================================================================
//   Copyright  2003-2004 by Sascha Willems - http://www.delphigl.de
// =============================================================================
//   --> visit the Delphi OpenGL Community - http://www.delphigl.com <--
// =============================================================================
//   Contents of this file are subject to the GNU Public License (GPL) which can
//   be obtained here : http://opensource.org/licenses/gpl-license.php
//   So only use this file if you fully unterstand that license!!!
// =============================================================================
//   Simple handling of pixelbuffers
// =============================================================================

unit glPBuffer;

interface

uses
 Windows,
 Classes,
 SysUtils,

 dglOpenGL;

type
 TPixelBuffer = class(TComponent)
  public
   Log       : TStringList;
   DC        : HDC;
   RC        : HGLRC;
   ParentDC  : HDC;
   ParentRC  : HGLRC;
   Handle    : TGLUInt;
   Width     : TGLUInt;
   Height    : TGLUInt;
   TextureID : TGLUInt;
   function IsLost : Boolean;
   procedure Enable;
   procedure Disable;
   procedure Bind;
   procedure Release;
   constructor Create(pWidth, pHeight : Integer;pParentDC, pParentRC : Cardinal;pAOwner : TComponent); reintroduce;
   destructor Destroy; override;
  end;

var
 PixelBuffer : TPixelBuffer;

implementation

constructor TPixelBuffer.Create(pWidth, pHeight : Integer;pParentDC, pParentRC : Cardinal;pAOwner : TComponent);
const
 PixelFormatAttribs  : array[0..12] of TGLUInt = (WGL_SUPPORT_OPENGL_ARB, GL_TRUE,
                                                  WGL_DRAW_TO_PBUFFER_ARB, GL_TRUE,
                                                  WGL_COLOR_BITS_ARB, 24,
                                                  WGL_ALPHA_BITS_ARB, 8,
                                                  WGL_DEPTH_BITS_ARB, 24,
                                                  WGL_DOUBLE_BUFFER_ARB, GL_FALSE, 0);

 PixelBufferAttribs  : array[0..4] of TGLUInt = (WGL_TEXTURE_FORMAT_ARB, WGL_TEXTURE_RGBA_ARB,
                                                 WGL_TEXTURE_TARGET_ARB, WGL_TEXTURE_2D_ARB, 0);
 EmptyF              : TGLFLoat = 0;
var
 PFormat       : array[0..64] of TGLUInt;
 NumPFormat    : TGLUInt;
 TempW, TempH  : TGLUInt;
 TempDC        : TGLUInt;
begin
inherited Create(pAOWner);
ParentDC := pParentDC;
ParentRC := pParentRC;
Width    := pWidth;
Height   := pHeight;
Log := TStringList.Create;
Log.Add('PixelBuffer->Creating->Width='+IntToStr(Width)+' Height='+IntToStr(Height));
TempDC := wglGetCurrentDC;
if TempDC > 0 then
 Log.Add('PixelBuffer->wglGetCurrentDC->Obtained valid DC : '+IntToStr(TempDC))
else
 begin
 Log.Add('PixelBuffer->wglGetCurrentDC->Couldn''t obtain valid device context');
 exit;
 end;
if wglChoosePixelFormatARB(TempDC, @PixelFormatAttribs, @EmptyF, Length(PFormat), @PFormat, @NumPFormat) then
 begin
 Log.Add('PixelBuffer->wglChoosePixelFormatARB->'+IntToStr(NumPFormat)+' suitable pixelformats found');
 end
else
 begin
 Log.Add('PixelBuffer->wglChoosePixelFormatARB->No suitable pixelformat found');
 exit;
 end;
Handle := wglCreatePBufferARB(TempDC, PFormat[0], Width, Height, @PixelBufferAttribs);
if Handle > 0 then
 begin
 wglQueryPbufferARB(Handle, WGL_PBUFFER_WIDTH_ARB, @TempW);
 wglQueryPbufferARB(Handle, WGL_PBUFFER_HEIGHT_ARB, @TempH);
 Log.Add('PixelBuffer->wglCreatePBufferARB->PixelBuffer successfully created.Received size : Width='+IntToStr(TempW)+' Height='+IntToStr(TempH));
 end
else
 begin
 Log.Add('PixelBuffer->wglCreatePBufferARB->Couldn''t obtain valid handle');
 exit;
 end;
DC := wglGetPBufferDCARB(Handle);
if DC > 0 then
 Log.Add('PixelBuffer->wglGetPBufferDCARB->Recieved valid DC for PBuffer : '+IntToStr(DC))
else
 begin
 Log.Add('PixelBuffer->wglGetPBufferDCARB->Couldn''t obtain valid DC for PBuffer');
 exit;
 end;
RC := wglCreateContext(DC);
if RC > 0 then
 Log.Add('PixelBuffer->wglCreateContext->Created rendercontext for PBuffer : '+IntToStr(RC))
else
 begin
 Log.Add('PixelBuffer->wglGetPBufferDCARB->Couldn''t create rendercontext for PBuffer');
 exit;
 end;
glGenTextures(1, @TextureID);
glBindTexture(GL_TEXTURE_2D, TextureID);
glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
end;

destructor TPixelBuffer.Destroy;
begin
wglDeleteContext(RC);
wglReleasePbufferDCARB(Handle, DC);
wglDestroyPbufferARB(Handle);
Log.Free;
inherited;
end;

function TPixelBuffer.IsLost : Boolean;
var
 Flag : TGLUInt;
begin
Result := False;
wglQueryPbufferARB(Handle, WGL_PBUFFER_LOST_ARB, @Flag);
if Flag <> 0 then
 Result := True;
end;

procedure TPixelBuffer.Enable;
begin
wglMakeCurrent(DC, RC);
end;

procedure TPixelBuffer.Disable;
begin
wglMakeCurrent(ParentDC, ParentRC);
end;

procedure TPixelBuffer.Bind;
begin
wglBindTexImageARB(Handle, WGL_FRONT_LEFT_ARB);
end;

procedure TPixelBuffer.Release;
begin
wglReleaseTexImageARB(Handle, WGL_FRONT_LEFT_ARB);
end;

end.
