unit main_stereo;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  dglOpenGl, knots, ExtCtrls, glPBuffer, Geometry, StdCtrls, primitives;

const
  mat_diffuse : Array[0..3] of TGlFloat =
    (0.8, 0.8, 0.8, 1.0);
  mat_specular : Array[0..3] of TGlFloat =
    (1.0, 1.0, 1.0, 1.0);
  mat_shininess : Array[0..0] of TGlFloat = (100.0);

type
  TColorSettings = Array[0..2] of Boolean;

  TOptions = record
               Stereo : Boolean;
               LeftColors  : TColorSettings;
               RightColors : TColorSettings;
             end;

  TStereoForm = class(TForm)
    Timer1: TTimer;
    GroupBox1: TGroupBox;
    Label1: TLabel;
    procedure FormCreate(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure FormResize(Sender: TObject);
    procedure FormMouseMove(Sender: TObject; Shift: TShiftState; X,
      Y: Integer);
    procedure FormMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure FormMouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure Timer1Timer(Sender: TObject);
    procedure FormKeyPress(Sender: TObject; var Key: Char);
  private
    { Private-Deklarationen }
    OpenGlInitialized : Boolean;
    DC                : HDC;
    RC                : HGLRC;
    Opt               : TOptions;
    PBuffer           : TPixelBuffer;

    XRotation, YRotation : Single;
    MouseX, MouseY : LongInt;

    zNear, zFar, Oeffnungswinkel, Augenabstand, Zielweite : Single;

    TorusKnot, Sphere : gluInt;

    //Daten, fr den Framecounter
    StartTick : Cardinal;
    Frames : LongInt;

    //Bewegung des Balls
    Pos, Direction : Extended;

    RenderTriangles : Integer;

    procedure ReadOptions;
    procedure Render;
    procedure Idle(Sender: TObject; var Done: Boolean);
  public
    { Public-Deklarationen }
  end;

var
  StereoForm: TStereoForm;

implementation

{$R *.DFM}

procedure InitGl;
begin
  glMaterialfv(GL_FRONT, GL_DIFFUSE, @mat_diffuse[0]);
  glMaterialfv(GL_FRONT, GL_SPECULAR, @mat_specular[0]);
  glMaterialfv(GL_FRONT, GL_SHININESS, @mat_shininess[0]);

  glEnable(GL_LIGHTING);
  glEnable(GL_LIGHT0);
  glEnable(GL_DEPTH_TEST); //Tiefenpuffer aktivieren

  glShadeModel(GL_SMOOTH);
end;

procedure TStereoForm.FormCreate(Sender: TObject);
begin
  OpenGlInitialized := False;
  ReadOptions;

  //Clipping Planes
  zNear := 1;
  zFar  := 100;
  //Sonst. Kameravoreinstellungen
  Oeffnungswinkel := 45;
  Augenabstand := 0.5;
  Zielweite := 40;
  Pos := 15;
  Direction := -0.1;

  Label1.Caption := Format('ffnung: %f Augenabstand: %f Zielweite %f', [Oeffnungswinkel, Augenabstand, Zielweite]);
end;

procedure TStereoForm.FormShow(Sender: TObject);

  function GetDesktopBits : Integer;
  var
    DesktopDC : HDC;
  begin
    //Farbtiefe des Desktops
    DesktopDC := GetDC(0);
    Result := GetDeviceCaps(DesktopDC, BITSPIXEL);
    if Result < 16 then
      Result := 16
    else if Result = 24 then
      Result := 32;

    ReleaseDC(0, DesktopDC);
  end;

begin
  if InitOpenGl then
  begin
    DC := GetDC(Self.Handle);
    RC := CreateRenderingContext(DC, [opDoubleBuffered], GetDesktopBits, 24, 0, 0, 0, 0);
    if (DC <> 0) and (RC <> 0) then
    begin
      ActivateRenderingContext(DC, RC);
      OpenGlInitialized := true;
      OnResize(Self);

      InitGl;

      //Torus Knot und Kugel erzeugen
      TorusKnot := glGenLists(1);
      glNewList(TorusKnot, GL_COMPILE);
        RenderTube;
      glEndList;

      Sphere := glGenLists(1);
      glNewList(Sphere, GL_COMPILE);
        BuildSphere(96,96);
      glEndList;

      //Idle Event
      Application.OnIdle := Idle;
    end
    else
    begin
      MessageDlg('Error creating Rendering Context!', mtError, [mbOk], 0);
      PostQuitMessage(0);
    end;
  end
  else
  begin
    MessageDlg('Error loading OpenGl Dlls!', mtError, [mbOk], 0);
    PostQuitMessage(0);
  end
end;

procedure TStereoForm.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  if Assigned(PBuffer) then
  begin
    PBuffer.Free;
    PBuffer := nil;
  end;

  DeactivateRenderingContext;
  wglDeleteContext(RC);
  ReleaseDC(Handle, DC);
end;

procedure TStereoForm.ReadOptions;
  function ReadParamColors(Param : String) : TColorSettings;
  var
    i : Integer;
  begin
    Result[0] := false; Result[1] := false; Result[2] := false;

    for i := 3 to Length(Param) do
    begin
      case Param[i] of
        'R': Result[0] := true;
        'G': Result[1] := true;
        'B': Result[2] := true;
      end
    end;
  end;

var
  I : Integer;
  Param, ParamPart : String;
begin
  Opt.Stereo := False;
  //grn frs linke Auge
  Opt.LeftColors[0] := false; Opt.LeftColors[1] := true; Opt.LeftColors[2] := true;
  //rot frs rechte Auge
  Opt.RightColors[0] := true; Opt.RightColors[1] := false; Opt.RightColors[2] := false;

  for i := 1 to ParamCount do
  begin
    Param := UpperCase(ParamStr(i));
    ParamPart := Param; SetLength(ParamPart, 2);
    if Param = '-STEREO' then
      Opt.Stereo := true
    else if ParamPart = '-L' then
    begin
      Opt.LeftColors := ReadParamColors(Param);
    end
    else if ParamPart = '-R' then
    begin
      Opt.RightColors := ReadParamColors(Param);
    end
  end;
end;

procedure TStereoForm.Render;
  procedure RenderScene;
  begin
    RenderTriangles := 0;

    //Vorderer Knoten
    glPushMatrix;
      glTranslatef(-2, -1, - 14.0);
      glRotatef(XRotation, 1, 0,0);
      glRotatef(YRotation, 0, 1,0);
      glCallList(TorusKnot);
    glPopMatrix;
    Inc(RenderTriangles, Knots.NumTriangles);

    //Hinterer Knoten
    glPushMatrix;
      glTranslatef(0, 0, - 25.0);
      glRotatef(XRotation+90, 1, 0,0);
      glRotatef(YRotation + 45, 0, 1,0);
      glCallList(TorusKnot);
    glPopMatrix;
    Inc(RenderTriangles, Knots.NumTriangles);

    //Bewegliche Kugel
    glPushMatrix;
      glTranslatef(3, Sin(Pos/4), -Pos);
      glCallList(Sphere);
    glPopMatrix;
    Inc(RenderTriangles, Primitives.Triangles);

    //Mittlere Kugel
    glPushMatrix;
      glTranslatef(-2, -1, - 19.0);
      glCallList(Sphere);
    glPopMatrix;
    Inc(RenderTriangles, Primitives.Triangles);
  end;

var
  Error : glUInt;
begin
  //Siehe Tutorial-Text!
  if (Opt.Stereo) and (Assigned(PBuffer)) then
  begin
    PBuffer.Enable;
    glClearColor(0.0, 0.0, 0.0, 0.0);
    glClear(GL_DEPTH_BUFFER_BIT or GL_COLOR_BUFFER_BIT);

    glLoadIdentity;
    glTranslatef(Augenabstand/2, 0, 0);

    glColorMask(Opt.LeftColors[0], Opt.LeftColors[1], Opt.LeftColors[2], true);
    RenderScene;
    PBuffer.Disable;

    glClearColor(0.0, 0.0, 0.0, 0.0);
    glClear(GL_DEPTH_BUFFER_BIT or GL_COLOR_BUFFER_BIT);

    glLoadIdentity;
    glTranslatef(-Augenabstand/2, 0, 0);

    glColorMask(Opt.RightColors[0], Opt.RightColors[1], Opt.RightColors[2], true);
    RenderScene;

    glColorMask(true, true, true, true);
    glDepthFunc(GL_ALWAYS);
    glMatrixMode(GL_PROJECTION);
      glPushMatrix;
      glLoadIdentity;
      glOrtho(0, 1, 1, 0, -1.0, 1.0);

      glMatrixMode(GL_MODELVIEW);
      glLoadIdentity;

      glDisable(GL_LIGHTING);
      glEnable(GL_TEXTURE_2D);
      PBuffer.Bind;
      glEnable(GL_BLEND);
      glBlendFunc(GL_ONE, GL_ONE);
      glBegin(GL_QUADS);
        glTexCoord2f(1, 0);
        glVertex2f(1,1);

        glTexCoord2f(1, 1);
        glVertex2f(1,0);

        glTexCoord2f(0, 1);
        glVertex2f(0,0);

        glTexCoord2f(0, 0);
        glVertex2f(0,1);
      glEnd();
      glDisable(GL_BLEND);

      glEnable(GL_LIGHTING);
      glDisable(GL_TEXTURE_2D);
      PBuffer.Release;

      glMatrixMode(GL_PROJECTION);
      glPopMatrix;
    glMatrixMode(GL_MODELVIEW);
    glDepthFunc(GL_LESS);
  end
  else
  begin
    glClearColor(0.0, 0.0, 0.0, 0.0);
    glClear(GL_DEPTH_BUFFER_BIT or GL_COLOR_BUFFER_BIT);

    glLoadIdentity;
    RenderScene;
  end;

  //Error Handler
  Error := glgetError;
  if Error <> GL_NO_ERROR then
  begin
    Caption := gluErrorString(Error);
    //Rendering kurz anhalten
    FlashWindow(Handle, True)
  end;

  //Frame Counter
  Inc(Frames);
  if GetTickCount - StartTick >=500 then
  begin
    Caption := Format('http://www.delphigl.com/ - Stereo Knot Viewer [%f FPS %d TRIs]',
      [Frames/(GetTickCount - StartTick)*1000, RenderTriangles]);
    Frames := 0;
    StartTick := GetTickCount
  end;
  SwapBuffers(DC);
end;

procedure TStereoForm.FormResize(Sender: TObject);
const
  PBufSize = 1024;
var
  SVerh, ROeffnung : Single;
  Breitenhaelfte, NeardZielweite : Single;
  left, right, top, bottom : Single;

  procedure CalcValues;
  begin
    ROeffnung := DegToRad(Oeffnungswinkel / 2);
    Breitenhaelfte := zNear * Tan(ROeffnung);
    NeardZielweite := zNear / Zielweite;
  end;

begin
  if Opt.Stereo then
  begin
    glViewport(0, 0, ClientWidth, ClientHeight);
    //Projektionsmatrix resetten
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();

    SVerh := ClientWidth/ClientHeight;
    CalcValues;
    left := - SVerh * Breitenhaelfte - 0.5 *Augenabstand*NeardZielweite;
    right := SVerh * Breitenhaelfte - 0.5 *Augenabstand*NeardZielweite;
    top := Breitenhaelfte;
    bottom := -Breitenhaelfte;
    glFrustum(left, right, bottom, top, zNear, zFar);

    glMatrixMode(GL_MODELVIEW);
    glLoadIdentity;

    if not Assigned(PBuffer) then
    begin
      PBuffer := TPixelBuffer.Create(PBufSize, PBufSize, DC, RC, Self);
      wglShareLists(RC, PBuffer.RC);
      PBuffer.Enable;
      InitGl;
    end
    else
      PBuffer.Enable;

    glViewport(0, 0, PBufSize, PBufSize);
    //Projektionsmatrix resetten
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();

    left := - SVerh * Breitenhaelfte + 0.5 *Augenabstand*NeardZielweite;
    right := SVerh * Breitenhaelfte + 0.5 *Augenabstand*NeardZielweite;
    top := Breitenhaelfte;
    bottom := -Breitenhaelfte;
    glFrustum(left, right, bottom, top, zNear, zFar);

    glMatrixMode(GL_MODELVIEW);
    glLoadIdentity;

    PBuffer.Disable;
  end
  else
  begin
    glViewport(0, 0, ClientWidth, ClientHeight);
    //Projektionsmatrix resetten
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    //Perspektivische Darstellung
    SVerh := ClientWidth/ClientHeight;
    CalcValues;
    left := - SVerh * Breitenhaelfte;
    right := SVerh * Breitenhaelfte;
    top := Breitenhaelfte;
    bottom := -Breitenhaelfte;
    glFrustum(left, right, bottom, top, zNear, zFar);

    glMatrixMode(GL_MODELVIEW);
    glLoadIdentity;

  end;
end;

procedure TStereoForm.Idle(Sender: TObject; var Done: Boolean);
begin
  Done := False;
  if OpenGlInitialized then Render;
end;

procedure TStereoForm.FormMouseMove(Sender: TObject; Shift: TShiftState; X,
  Y: Integer);
begin
  if MouseCapture then
  begin
    YRotation := YRotation + ((MouseX - X) div 2);
    XRotation := XRotation + ((MouseY - Y) div 2);
    MouseX := X;
    MouseY := Y
  end
end;

procedure TStereoForm.FormMouseDown(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
  MouseCapture := True;
  MouseX := X;
  MouseY := Y
end;

procedure TStereoForm.FormMouseUp(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
  MouseCapture := false;
end;

procedure TStereoForm.Timer1Timer(Sender: TObject);
begin
  //Bewegung der Kugel und Drehung der Objekte
  YRotation := YRotation + 0.5;
  Pos := Pos + Direction;
  if Pos < 9 then
  begin
    Direction := 0.3;
  end
  else if Pos > 60 then
  begin
    Direction := - 0.3;
  end;
end;

procedure TStereoForm.FormKeyPress(Sender: TObject; var Key: Char);
begin
  //Tastaturereignisse
  case Key of
    's': begin
           Opt.Stereo := not Opt.Stereo;
         end;
    'd': begin
           Oeffnungswinkel := Oeffnungswinkel + 1.0
         end;
    'c': begin
           Oeffnungswinkel := Oeffnungswinkel - 1.0
         end;
    'f': begin
           Augenabstand := Augenabstand + 0.005
         end;
    'v': begin
           Augenabstand := Augenabstand - 0.005
         end;
    'g': begin
           Zielweite := Zielweite + 0.5
         end;
    'b': begin
           Zielweite := Zielweite - 0.5
         end;
    'r': begin
           Oeffnungswinkel := 45;
           Augenabstand := 0.5;
           Zielweite := 40;
         end;
    't': begin
           GroupBox1.Visible := not GroupBox1.Visible;
         end;
  end;
  OnResize(Self);
  Label1.Caption := Format('ffnung: %f Augenabstand: %f Zielweite %f', [Oeffnungswinkel, Augenabstand, Zielweite]);
end;

end.
